use strict;

use Net::POP3;

our $HOST = "mail.investordna.net";
our $USER = "incoming+investordna.net";
our $PASSWORD = ',*H$hWE}|.Z*';

our $OUT_DIR = "/home1/investq6/tmp/parsing/";

# Constructors
my $pop = Net::POP3->new($HOST);
$pop = Net::POP3->new($HOST, Timeout => 60);
open (TEST, ">" . $OUT_DIR . "msgDump.txt");
if ($pop->login($USER, $PASSWORD) > 0) {
  my $msgnums = $pop->list; # hashref of msgnum => size
  foreach my $msgnum (keys %$msgnums) {
    my $msg = $pop->get($msgnum);
    my $output;
    my $source;
    foreach my $msg_line (@$msg) {
    	print TEST $msg_line . "\n";
    	#reuters
	if ($msg_line =~ /From: Reuters\.com/) {
    		$output = parse_reuters($msg);
    		$source = 'reuters';
    	}	
    }
    #google alert	
    $output = parse_google_alert($msg);
    $source = 'google_alert' if ($output);
    #marketwatch
    $output = parse_marketwatch_alert($msg);
    $source = 'marketwatch_alert' if ($output);
    #hedgeworld
    $output = parse_hedgeworld($msg);
    $source = 'hedgeworld' if ($output);
	
    writeOutput($source, $output) if ($output);		
    #$pop->delete($msgnum);
  }
}

$pop->quit;

sub parse_reuters {
	my $msg = shift;
	my %output = ();
	my $parsing_data = 0;
	my $entry_line_count = 0;
	my $entry_count = 0;
	foreach my $line (@$msg) {
		$parsing_data = 0 if ($line =~ /:$/);
		$line =~ s/>\s+//g;
		chomp $line;
		if ($parsing_data) {
			
			if (!$line) {
				$entry_line_count = 0;
				$entry_count++;
			}	
			if ($entry_line_count == 1) {
				print "Reader header $line ...\n";
				$output{$entry_count}->{header} = $line  
			}	
			if ($entry_line_count > 1) {
				if ($line =~ /http:/) {
					
					$output{$entry_count}->{link} = $line;
				} else {	
					$output{$entry_count}->{text} .= $line . "\n";
				}	
			}
			$entry_line_count++;			
		}
		if ($line =~ /Good Morning,$/) {
			$parsing_data = 1 ;
			print $line;
		}	
	}
	return \%output;
}				

sub parse_google_alert {
	my $msg = shift;
	my $full_text = join " ", @$msg;
	return unless ($full_text =~ /Google News Alert/);
	$full_text =~ s/\n/ /g;
	my %output = ();
	my $entry_count = 0;
	if ($full_text =~ /Google News Alert for:.*?\*\s+(.*?)<(http:.*?)>(.*?)\*\.\.\.\*/s) {
		$output{$entry_count}->{header} = $1;
		$output{$entry_count}->{link} = $2;
		$output{$entry_count}->{text} = $3;
		$entry_count++;
	}
	while ($full_text =~ /See all stories.*?\>\s+(.*?)<(http:.*?)>(.*?)\*\.\.\.\*/gs) {
		$output{$entry_count}->{header} = $1;
		$output{$entry_count}->{link} = $2;
		$output{$entry_count}->{text} = $3;
		$entry_count++;
	}
	return \%output;
}			

sub parse_marketwatch_alert {
	my $msg = shift;
	my $full_text = join " ", @$msg;
	return unless ($full_text =~ /Alerts<alerts\@marketwatchmail\.com>/);
	$full_text =~ s/\n/ /g;
	my %output = ();
	my $entry_count = 0;
	if ($full_text =~ /ALERT \(KEYWORD:.*?"\)\*\s+\*(.*?)\*\s+<(http:.*?)>/s) {
		$output{$entry_count}->{header} = $1;
		$output{$entry_count}->{link} = $2;
		$entry_count++;
	}
	while ($full_text =~ /--------------\s+.*?\*(.*?)\*\s+<(http:.*?)>.*?-----/gs) {
		$output{$entry_count}->{header} = $1;
		$output{$entry_count}->{link} = $2;
		$entry_count++;
	}
	return \%output;
}			

sub parse_hedgeworld {
	my $msg = shift;
	my $full_text = join " ", @$msg;
	return unless ($full_text =~ /alt_advantage_html\@hedgeworld.com/);
	$full_text =~ s/\n/ /g;
	my %output = ();
	my $entry_count = 0;
	$full_text =~ s/^.*?Headlines from today:*//;
	while ($full_text =~ /\*(.*?)<(http:.*?)>\s+\*(.*?)\.\.\.\.\.\./gs) {
		$output{$entry_count}->{header} = $1;
		my $link = $2;
		$link =~ s/=\s+//g;
		$output{$entry_count}->{link} = $link;
		my $text = $3;
		$text =~ s/=\s+//g;
		$output{$entry_count}->{text} = $text;
		$entry_count++;
	}
	return \%output;
}	

sub writeOutput {
	my ($source, $output) = @_;
	
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
                                                localtime(time);

	$year += 1900;
	$mon += 1;
	$mon = '0' . $mon if ($mon < 10);
	$mday = '0' . $mday if ($mday < 10);	

	mkdir $OUT_DIR . "$source";
	my $file = $OUT_DIR . $year . $mon . $mday . "/" . $source . ".xml";
	open (OUT, ">" . $file) or die "Could not open $file for writing...\n";
	print OUT qq(<?xml version="1.0" encoding="UTF-8"?>\n);
	print OUT qq(<entries>\n);
	foreach my $entry_id (sort {$a <=> $b} keys %$output) {
		print "Writing entry $entry_id...\n";
		my $entry = $output->{$entry_id};
		print "Writing header $entry->{header}...\n";
		print OUT qq(<entry>\n);
		print OUT qq(  <source>$source</source>\n);
		print OUT qq(  <header><![CDATA[$entry->{header}]]></header>\n);
		print OUT qq(  <link><![CDATA[$entry->{link}]]></link>\n);
		print OUT qq(  <text><![CDATA[$entry->{text}]]></text>\n);
		print OUT qq(</entry>\n);
	}
	print OUT qq(</entries>\n);
	close OUT;
}		

